#!/usr/bin/perl -w

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use lib '..';

use strict;
use vars qw($db);

use Carp::Datum (":all", defined($ENV{'GTDEBUG'}) ? "on" : "off");
use GT::Prices;
use GT::Portfolio;
use GT::PortfolioManager;
use GT::Calculator;
use GT::Report;
use GT::BackTest;
use GT::Eval;
use Getopt::Long;
use GT::Conf;
use GT::DateTime;
use GT::Tools qw(:conf);

GT::Conf::load();

=head1 ./backtest.pl [ options ] <code>

=head1 ./backtest.pl <system_alias> <code>

=head2 Description

Backtest will run a backtest of the system called systemname
(available as GT::Systems::<systemname>) on share of indicated code.

You can either describe the full system with all the options, or you can
give an alias of a system. The alias is set your configuration file with
entries like "Aliases::<alias_name> <full_system_name>". The full
system name is like "SY:TFS|CS:SY:TFS|CS:Stop:Fixed 4|MM:VAR".

=head2 Options

Backtest provide a set of options, so that you can use a combination
of MoneyManagement, TradeFilters, OrderFactory an CloseStrategy modules.

=over 4

=item --full 

Runs the backtest on the full history (it runs on two years by default) 

=item --timeframe="day|week|month|year"

Launch the system while using the indicated timeframe.

=item --system="<system_name>"

use the GT::Systems::<system_name> as the source of buy/sell orders.  

=item --money-management="<money_management_name>" 

use the GT::MoneyManagement::<money_management_name> as money management system.

=item --trade-filter="<filter_name>"

use the GT::TradeFilters::<filter_name> as a trade filter.  

=item --order-factory="<order_factory_name>" 

use GT::OrderFactory::<order_factory_name> as an order factory.  

=item --close-strategy="<close_strategy_name>" 

use GT::CloseStrategy::<close_strategy_name> as a close strategy.

=back

=head2 Examples

=over 4

=item

./backtest.pl TFS 13000

=item

./backtest.pl --full TFS 13000

=item

./backtest.pl --close-strategy="Systems::TFS" --close-strategy="Stop::Fixed 6" --money-management="VAR" --money-management="SizeLimit" --system="TFS" --broker="SelfTrade Intgral" 13000

=back

=cut

# Gestion des options
my ($full, $verbose, $ofname, $broker, $system, $timeframe, $start, $end) = 
   (0, 0, '', '', '', '', '', '');
my (@mmname, @tfname, @csname);
GetOptions('full!' => \$full, 'verbose' => \$verbose, 
	   'money-management=s' => \@mmname,
	   'trade-filter=s' => \@tfname, 'order-factory=s' => \$ofname,
	   'close-strategy=s' => \@csname, 'broker=s' => \$broker,
	   'system=s' => \$system, "timeframe=s" => \$timeframe,
	   'start=s' => \$start, 'end=s' => \$end);

if (! scalar(@mmname))
{
    @mmname = ("Basic");
}
if ($system && ! scalar(@csname)) {
    die "You must give at least one --close-strategy argument !\n";
}

# Il faut crer tout le framework
my $db = create_standard_object("DB::" . GT::Conf::get("DB::module"));
my $pf_manager = GT::PortfolioManager->new;
my $sys_manager = GT::SystemManager->new;


if ($system) {
    $sys_manager->set_system(
	    create_standard_object(split (/\s+/, "Systems::$system")));
} else {
    my $alias = shift;
    if (! defined($alias)) {
	die "You must give either a --system parameter or an alias name.";
    }
    my $sysname = resolve_alias($alias);
    if (defined($sysname) && $sysname)
    {
	$sys_manager->setup_from_name($sysname);
	$pf_manager->setup_from_name($sysname);
    } else {
	die "You must give either a --system parameter or an alias name.";
    }
}

foreach (@mmname)
{
    $pf_manager->add_money_management_rule(
	create_standard_object(split (/\s+/, "MoneyManagement::$_")));
}
$pf_manager->default_money_management_rule(
	create_standard_object("MoneyManagement::Basic"));
$pf_manager->finalize;

if ($ofname)
{
    $sys_manager->set_order_factory(
	create_standard_object(split (/\s+/, "OrderFactory::$ofname")));
}
foreach (@tfname)
{
    $sys_manager->add_trade_filter(
	create_standard_object(split (/\s+/, "TradeFilters::$_")));
}
foreach (@csname)
{
    $sys_manager->add_position_manager(
	create_standard_object(split (/\s+/, "CloseStrategy::$_")));
}
$sys_manager->finalize;

# Hop au boulot prliminaire sur les valeurs  backtester
my $code = shift;
if (! $code) {
    die "You must give a symbol for the simulation.\n";
}
my $q = $db->get_prices($code);

my $calc = GT::Calculator->new($q);
$calc->set_code($code);

if ($timeframe)
{
    if (! $calc->set_current_timeframe(
	    GT::DateTime::name_to_timeframe($timeframe)))
    {
	die "Can't create  $timeframe  timeframe ...\n";
    }
}

my $c = $calc->prices->count;
my $last = $c - 1;
my $first = $c - 2 * GT::DateTime::timeframe_ratio($YEAR, 
						   $calc->current_timeframe);
$first = 0 if ($full);
$first = 0 if ($first < 0);
if ($start) {
    my $date = $calc->prices->find_nearest_following_date($start);
    $first = $calc->prices->date($date);
}
if ($end) {
    my $date = $calc->prices->find_nearest_preceding_date($end);
    $last = $calc->prices->date($date);
}

# Au boulot
my $analysis = backtest_single($pf_manager, $sys_manager, $broker, $calc, $first, $last);

# Affichage des rsultats
print "## Analysis of " . $sys_manager->get_name . "|" .
	$pf_manager->get_name . "\n";
GT::Report::Portfolio($analysis->{'portfolio'}, $verbose);

print "## Global analysis (each position is 10keuros, value of portfolio)\n";
GT::Report::PortfolioAnalysis($analysis->{'real'}, $verbose);
print "\n## Theoretical analysis (10keuros, full portfolio reinvested)\n";
GT::Report::PortfolioAnalysis($analysis->{'theoretical'}, $verbose);

$db->disconnect;

