package GT::CloseStrategy::Stop::ExtremePrices;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::CloseStrategy;
use GT::Prices;
use Carp::Datum;
use GT::Indicators qw($GET_LOW $GET_HIGH);
use GT::Indicators::Generic::MinInPeriod;
use GT::Indicators::Generic::MaxInPeriod;

@ISA = qw(GT::CloseStrategy);
@NAMES = ("ExtremePrices[#1,#2]");

=head1 GT::CloseStrategy::Stop::ExtremePrices

This strategy closes the position once the prices have crossed up the
highest high in a short position and crossed down the highest low in a
logn position.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [ 20, 1 ] };

    $args->[0] = 20 if (! defined($args->[0]));
    $args->[1] = 1 if (! defined($args->[1]));
	
    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my ($self) = @_;

    $self->{'min'} = GT::Indicators::Generic::MinInPeriod->new([ $self->{'args'}[0] ], "LOW", $GET_LOW);
    $self->{'max'} = GT::Indicators::Generic::MaxInPeriod->new([ $self->{'args'}[0] ], "HIGH", $GET_HIGH);
    
    $self->add_indicator_dependency($self->{'min'}, 1);
    $self->add_indicator_dependency($self->{'max'}, 1);
    $self->add_prices_dependency($self->{'args'}[0]);
}

sub get_indicative_long_stop {
    my ($self, $calc, $i, $order, $pf_manager, $sys_manager) = @_;
    my $percentage = $self->{'args'}[1];

    return if (! $self->check_dependencies($calc, $i));

    my $lowest_low = $calc->indicators->get($self->{'min'}->get_name, $i);

    return ($lowest_low * (1 - $percentage / 100));
}

sub get_indicative_short_stop {
    my ($self, $calc, $i, $order, $pf_manager, $sys_manager) = @_;
    my $percentage = $self->{'args'}[1];
    
    return if (! $self->check_dependencies($calc, $i));

    my $highest_high = $calc->indicators->get($self->{'max'}->get_name, $i);
    
    return ($highest_high * (1 + $percentage / 100));
}

sub long_position_opened {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    return;
}

sub short_position_opened {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    return;
}

sub manage_long_position {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;
    my $percentage = $self->{'args'}[1];

    return if (! $self->check_dependencies($calc, $i));
 
    my $lowest_low = $calc->indicators->get($self->{'min'}->get_name, $i);
    $position->set_stop($lowest_low * (1 - $percentage / 100));
    
    return;
}

sub manage_short_position {
    my ($self, $calc, $i, $position, $ps_manager, $sys_manager) = @_;
    my $percentage = $self->{'args'}[1];

    return if (! $self->check_dependencies($calc, $i));
 
    my $highest_high = $calc->indicators->get($self->{'max'}->get_name, $i);
    $position->set_stop($highest_high * (1 + $percentage / 100));
    
    return;
}

