package GT::CloseStrategy::Stop::KeepRunUp;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::CloseStrategy;
use GT::Indicators qw($GET_LOW $GET_HIGH);
use GT::Indicators::Generic::MaxInPeriod;
use GT::Indicators::Generic::MinInPeriod;
use GT::Prices;
use Carp::Datum;

@ISA = qw(GT::CloseStrategy);
@NAMES = ("KeepRunUp[#1]");

=head1 GT::CloseStrategy::Stop::KeepRunUp

=head2 Overview

This strategy closes the position once the prices have crossed the
trailing stop defined as a percentage below the highest high value for a
long trade or above the highest low value for a short trade, called the
"run up", since the trade is open. The purpose of this strategy is to keep
opening profits and avoid to turn profitable trades into loosing ones.

=cut

sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my $args = shift;

    my $self = { "args" => defined($args) ? $args : [ 10 ] };

    $args->[0] = 10 if (! defined($args->[0]));

    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub get_indicative_long_stop {
    my ($self, $calc, $i, $order, $pf_manager, $sys_manager) = @_;
    my $percentage = $self->{'args'}[0];
    
    return ($calc->prices->at($i)->[$LAST] * (1 - $percentage / 100));
}

sub get_indicative_short_stop {
    my ($self, $calc, $i, $order, $pf_manager, $sys_manager) = @_;
    my $percentage = $self->{'args'}[0];

    return ($calc->prices->at($i)->[$LAST] * (1 + $percentage / 100));
}

sub long_position_opened {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    return;
}

sub short_position_opened {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;

    return;
}

sub manage_long_position {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;
    my $percentage = $self->{'args'}[0];
    my $date = $position->open_date;
    my $code = $position->code;
    my $prices = $calc->prices;
    
    if ($i >= $prices->date($date)) {

	my $max = GT::Indicators::Generic::MaxInPeriod->new([$date], "HIGH", $GET_HIGH);
        $max->calculate($calc, $i);
	
        my $highest_high = $calc->indicators->get($max->get_name, $i);
	$position->set_stop($highest_high * (1 - $percentage / 100));
    }

    return;
}

sub manage_short_position {
    my ($self, $calc, $i, $position, $pf_manager, $sys_manager) = @_;
    my $percentage = $self->{'args'}[0];
    my $date = $position->open_date;
    my $code = $position->code;
    my $prices = $calc->prices;

    if ($i >= $prices->date($date)) {

	my $min = GT::Indicators::Generic::MinInPeriod->new([$date], "LOW", $GET_LOW);
	$min->calculate($calc, $i);

        my $lowest_low = $calc->indicators->get($min->get_name, $i);
	$position->set_stop($lowest_low * (1 + $percentage / 100));
    }

    return;
}

1;
