package GT::Indicators::EMA;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

# Standards-Version: 1.0

use strict;
use vars qw(@ISA @NAMES @DEFAULT_ARGS);

use GT::Indicators;
use GT::Indicators::SMA;

@ISA = qw(GT::Indicators);
@NAMES = ("EMA[#*]");
@DEFAULT_ARGS = (20, "{I:Prices CLOSE}");

=head1 NAME

GT::Indicators::EMA - Exponential Moving Average

=head1 DESCRIPTION 

An exponential moving average gives more importance to
recent prices ...

=head2 Parameters

=over 

=item Period (default 20)

The first argument is the period used to calculed the average.

=item Other data input

The second argument is optional. It can be used to specify an other
stream of input data for the average instead of the close prices.
This is usually an indicator (detailed via {I:MyIndic<param>}).

=back

=head2 Creation

 GT::Indicators::EMA->new()
 GT::Indicators::EMA->new([20])

If you need a 30 days EMA of the opening prices you can write
one of those lines :

 GT::Indicators::EMA->new([30], "OPEN", $GET_OPEN)
 GT::Indicators::EMA->new([30, "{I:Prices OPEN}"])

A 10 days EMA of the RSI could be created with :

 GT::Indicators::EMA->new([10, "{I:RSI}"])

Z<>

=cut
sub initialize {
    my ($self) = @_;

	$self->{'sma'} = GT::Indicators::SMA->new([$self->{'args'}->get_arg_constant(1)], "SMA", $self->{'func'});
	$self->add_indicator_dependency($self->{'sma'}, $self->{'args'}->get_arg_constant(1));
	$self->add_prices_dependency($self->{'args'}->get_arg_constant(1));
}

sub calculate {
    my ($self, $calc, $i) = @_;
    my $nb = $self->{'args'}->get_arg_values($calc, $i, 1);
    my $getvalue = $self->{'func'};
    my $name = $self->get_name;
    
    return if (! defined($nb));
    
    $self->remove_volatile_dependencies();
    $self->add_volatile_arg_dependency(2, $nb);
    
    return if ($calc->indicators->is_available($name, $i));
    return if (! $self->check_dependencies($calc, $i));

    my $alpha = 2 / ($nb + 1);
	
	my $ema = &$getvalue($calc, $i - $nb + 1);
	
	if ($calc->indicators->is_available($name, $i-$nb))
	{
		$ema = $calc->indicators->get($name, $i-$nb);
	} 
	elsif ($i >= 4*$nb - 1)
	{			
		
		my @ema = ();
		for (my $bar = $i - 3*$nb; $bar <= $i-$nb; $bar++)
		{		
			if (defined ($ema[$bar-$nb]))
			{
				$ema = $ema[$bar-$nb];
			}
			else 
			{
				# This may look a bit dodgy but if the next line is omitted, SMA.pm uses calculate_interval
				# which fails in this context. Actually, this might be a bug in SMA.pm.
				$self->{'sma'}->calculate($calc, $bar - $nb);
				$ema = $calc->indicators->get($self->{'sma'}->get_name(0), $bar-$nb);
			}
			for(my $n = $bar - $nb + 1; $n <= $bar; $n++) 
    		{
       			$ema *= (1 - $alpha);
       			$ema += ($alpha * &$getvalue($calc, $n));
    		}
			$ema[$bar] = $ema;
		}
	}
	elsif ($i >= 2*$nb - 1)	{				
		
		$self->{'sma'}->calculate($calc, $i - $nb);
		$ema = $calc->indicators->get($self->{'sma'}->get_name(0), $i-$nb);
	}	
	
    return if (! defined($ema));
	
	for(my $n = $i - $nb + 1; $n <= $i; $n++) 
    {
       $ema *= (1 - $alpha);
       $ema += ($alpha * &$getvalue($calc, $n));
    }
	
    $calc->indicators->set($name, $i, $ema);
}
