package GT::Indicators::SMA;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

# Standards-Version: 1.0

use strict;
use vars qw(@ISA @NAMES @DEFAULT_ARGS);

use GT::Indicators;

@ISA = qw(GT::Indicators);
@NAMES = ("SMA[#*]");
@DEFAULT_ARGS = (50, "{I:Prices CLOSE}");

=head1 NAME

GT::Indicators::SMA - Simple Moving Average

=head1 DESCRIPTION 

A simple arithmetic moving average.

=head2 Parameters

=over 

=item Period (default 50)

The first argument is the period used to calculed the average.

=item Other data input

The second argument is optional. It can be used to specify an other
stream of input data for the average instead of the close prices.
This is usually an indicator (detailed via {I:MyIndic| <param>})
but it can also be "OPEN", "HIGH", "LOW", "CLOSE", "FIRST" and
"LAST" and in which cases the corresponding prices serie will be
used.

=back

=head2 Creation

 GT::Indicators::SMA->new()
 GT::Indicators::SMA->new([20])

If you need a 30 days SMA of the opening prices you can write
one of those lines :

 GT::Indicators::SMA->new([30], "OPEN", $GET_OPEN)
 GT::Indicators::SMA->new([30, "OPEN"])

A 10 days SMA of the RSI could be created with :

 GT::Indicators::SMA->new([10, "{I:RSI}"])
 
=cut
sub initialize {
    my ($self) = @_;
}

sub calculate {
    my ($self, $calc, $i) = @_;
    my $nb = $self->{'args'}->get_arg_values($calc, $i, 1);
    my $getvalue = $self->{'func'};
    my $name = $self->get_name;
    my $sum = 0;

    return if (! defined($nb));

    $self->remove_volatile_dependencies();
    $self->add_volatile_prices_dependency($nb);

    return if (! $self->check_dependencies($calc, $i));
    
    for(my $n = $i - $nb + 1; $n <= $i; $n++) 
    {
	my $val = &$getvalue($calc, $n);
	return if (! defined($val));
	$sum += $val;
    }
    $calc->indicators()->set($name, $i, $sum / $nb);
}

sub calculate_interval {
    my ($self, $calc, $first, $last) = @_;
    my $name = $self->get_name;

    ($first, $last) = $self->update_interval($calc, $first, $last);
    return if ($calc->indicators->is_available_interval($name, $first, $last));
    return if (! $self->check_dependencies_interval($calc, $first, $last));
    
    if ($self->{'args'}->is_constant(1)) {
	my $nb = $self->{'args'}->get_arg_constant(1);
	my $sum = 0;
	my $getvalue = $self->{'func'};
	# Calculate the new SMA value from the previous one when possible
	for(my $i = $first - $nb + 1; $i <= $last; $i++)
	{
	    my $val = &$getvalue($calc, $i);
	    next if (! defined($val));
	    if ($i <= $first) {
		$sum += $val;
		if ($i == $first) {
		    $calc->indicators->set($name, $i, $sum / $nb);
		}
	    } else {
		$sum += $val - &$getvalue($calc, $i - $nb);
		$calc->indicators->set($name, $i, $sum / $nb);
	    }
	}
    } else {
	# Calculate for each day separately
	GT::Indicators::calculate_interval(@_);
    }
}
