package GT::Indicators::VHF;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA @NAMES);

use GT::Indicators;
use GT::Indicators::Generic::MinInPeriod;
use GT::Indicators::Generic::MaxInPeriod;
use GT::Prices;

@ISA = qw(GT::Indicators);
@NAMES = ("VHF[#1]");

=head1 GT::Indicators::VHF

=head2 Overview

The Vertical Horizontal Filter (VHF) can tell you whether a market is going through a trending or congestion phase, and whether you should use trend-following indicators if the markets are trending or congestion-phase indicators if markets are in a trading range.

=head2 Calculation

VHF = (Highest Close (n) - Lowest Close (n)) / (Sum of absolute value of the one-day price change for the range (n))

=head2 Parameters

The standard VHF works with a 28-days parameter : n = 28

=head2 Example

GT::Indicators::VHF->new()
GT::Indicators::VHF->new([50])

=head2 Links

http://www.equis.com/free/taaz/verthorizfilter.html
http://www.finance-net.com/apprendre/techniques/vhf.phtml

=cut
sub new {
    my $type = shift;
    my $class = ref($type) || $type;
    my ($args) = @_;
    my $self = { 'args' => defined($args) ? $args : [28] };

    $args->[0] = 28 if (! defined($args->[0]));
       
    return manage_object(\@NAMES, $self, $class, $self->{'args'}, "");
}

sub initialize {
    my $self = shift;

    $self->{'min'} = GT::Indicators::Generic::MinInPeriod->new([ $self->{'args'}[0] ], "LAST", $GET_LAST);
    $self->{'max'} = GT::Indicators::Generic::MaxInPeriod->new([ $self->{'args'}[0] ], "LAST", $GET_LAST);

    $self->add_indicator_dependency($self->{'min'}, 1);
    $self->add_indicator_dependency($self->{'max'}, 1);
    $self->add_prices_dependency($self->{'args'}[0]);
}

=head2 GT::Indicators::VHF::calculate($calc, $day)

=cut
sub calculate {
    my ($self, $calc, $i) = @_;
    my $indic = $calc->indicators;
    my $prices = $calc->prices;
    my $period = $self->{'args'}[0];
    my $min = $self->{'min'};
    my $max = $self->{'max'};
    my $min_name = $min->get_name;
    my $max_name = $max->get_name;
    my $vhf_name = $self->get_name(0);
    my $sum_of_one_day_price_change = 0;

    return if ($indic->is_available($vhf_name, $i));
    return if (! $self->check_dependencies($calc, $i));
    
    # Get Highest Close and Lowest Close values
    my $lowest_close = $indic->get($min_name, $i);
    my $highest_close = $indic->get($max_name, $i);

    for (my $n = $i - $period + 1; $n <= $i; $n++) {
        # Calculate the sum of absolute value of the one-day price change over the period
        $sum_of_one_day_price_change += abs(($prices->at($n)->[$LAST] - $prices->at($n - 1)->[$LAST]));

    }
    
    # Calculate the VHF
    my $vhf = ($highest_close - $lowest_close) / $sum_of_one_day_price_change;
 
    # Return the results
    $indic->set($vhf_name, $i, $vhf);
}

1;
