package GT::Serializer::XML;

# Copyright 2000-2002 Raphal Hertzog, Fabien Fulhaber
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA);

use GT::Prices;
use GT::Portfolio;
use GT::MetaInfo;
use GT::Serializer;
use GT::DateTime;

@ISA = qw(GT::Serializer);

=head2 $serializer->store_prices($file, $prices)

Store all data of a GT::Prices object in a specified XML file.

=cut

sub store_prices {
    my ($self, $file, $prices) = @_;

    my $xml = GT::MetaInfo->new;
    
    foreach (@{$prices->{'prices'}}) {
        $xml->set("prices/record", "", {
	    "id" => $prices->{'dates'}{$_->[$DATE]},
	    "open" => $_->[$OPEN],
	    "high" => $_->[$HIGH],
	    "low" => $_->[$LOW],
	    "close" => $_->[$CLOSE],
	    "volume" => $_->[$VOLUME],
	    "date" => $_->[$DATE]});
    }
    if (defined($prices->timeframe)) {
	$xml->set("prices/timeframe", $prices->timeframe);
    }
    $xml->set("prices/nb_records", $prices->count);

    $xml->save($file);
}

=head2 $serializer->load_prices($file)

Load an XML file and return a GT::Prices object.

=cut

sub load_prices {
    my ($self, $file) = @_;
    
    my $prices = GT::Prices->new;
    my $xml = GT::MetaInfo->new;
    $xml->load($file);

    foreach ($xml->list("prices/record")) {
	$prices->add_prices([$_->{'open'}, $_->{'high'}, $_->{'low'}, $_->{'close'}, $_->{'volume'}, $_->{'date'}]);
    }
    
    $prices->sort();
    $prices->reindex();
    
    my $timeframe = $xml->get("prices/timeframe");
    if (defined($timeframe) && $timeframe) {
	$prices->set_timeframe($timeframe);
    } else {
	$prices->set_timeframe($DAY);
    }
    return $prices;
}

### The following code is NOT yet compatible with GT latest portfolio
### structure.

sub store_portfolio {
    my ($self, $file, $p) = @_;
    
    my $xml = GT::MetaInfo->new;
    foreach (grep { defined($_) } @{$p->{'pending_orders'}})
    {
	$xml->set("portfolio/pending_orders/order", "", $_);
    }
    foreach (grep { defined($_) } @{$p->{'discarded_orders'}})
    {
	$xml->set("portfolio/discarded_orders/order", "", $_);
    }
    foreach (grep { defined($_) } @{$p->{'open_positions'}})
    {
	$xml->set("portfolio/open_positions/position", "", $_);
	#	    { "uniq_id" => $_->{'id'} });
    }
    foreach (grep { defined($_) } @{$p->{'history'}})
    {
	$xml->set("portfolio/history/position", "", $_);
	#	    { "uniq_id" => $_->{'id'} });
    }
    foreach (grep { defined($_) } values %{$p->{'positions'}})
    {
	$xml->set("portfolio/positions/position", "", $_);
	
	foreach my $d (grep { defined($_) }
		      @{$p->{'position_details'}{$_->{'id'}}})
	{
	    $xml->set("portfolio/position_details/order", "", $d);
	}
	foreach my $o (grep { defined($_) }
		      @{$p->{'position_orders'}{$_->{'id'}}})
	{
	    $xml->set("portfolio/position_orders/order", "", $o);
	}
    }
    $xml->set("portfolio/nb_positions", $p->{'global_position_id'});

    $xml->save($file);
}

sub load_portfolio {
    my ($self, $file) = @_;

    my $p = GT::Portfolio->new;
    my $xml = GT::MetaInfo->new;
    $xml->load($file);

    foreach ($xml->list("portfolio/pending_orders/order"))
    {
	$p->add_order($_);
    }
    foreach ($xml->list("portfolio/discarded_orders/order"))
    {
	push @{$p->{'discarded_orders'}}, $_;
    }

    foreach ($xml->list("portfolio/positions/position"))
    {
	$p->{'positions'}{$_->{'uniq_id'}} = $_;
	$p->{'position_orders'}{$_->{'uniq_id'}} = [];
	$p->{'position_details'}{$_->{'uniq_id'}} = [];
    }

    foreach ($xml->list("portfolio/open_positions/position"))
    {
	$p->{'positions'}{$_->{'uniq_id'}}{'id'} = 
	    scalar(@{$p->{'open_positions'}});
	push @{$p->{'open_positions'}}, 
	     $p->{'positions'}{$_->{'uniq_id'}};
    }
    $p->{'position_id'} = scalar(@{$p->{'open_positions'}});
    
    foreach ($xml->list("portfolio/history/position"))
    {
	push @{$p->{'history'}}, $p->{'positions'}{$_->{'uniq_id'}};;
    }

    foreach ($xml->list("portfolio/position_details/order"))
    {
	push @{$p->{'positions_details'}{$_->{'position'}}}, $_;
    }
    foreach ($xml->list("portfolio/position_orders/order"))
    {
	push @{$p->{'positions_orders'}{$_->{'position'}}}, $_;
    }
    
    $p->{'global_position_id'} = $xml->get("portfolio/nb_positions");
    return $p;
}

1;
