package GT::DB::bean;

# Copyright 2003 Sai-kee Wong
# This file is distributed under the terms of the General Public License
# version 2 or (at your option) any later version.

use strict;
use vars qw(@ISA);

use GT::DB;
use GT::Prices;
use GT::Conf;
use GT::DateTime;

use DBD::mysql;

@ISA = qw(GT::DB);

=head1 NAME

GT::DB::bean - Access to beancounter database of quotes

=head1 DESCRIPTION

This module is used to retrieve quotes from a MySQL database
as setup by beancounter.
By default, the database is supposed to be running on localhost and
the only authentication done is the standard Unix one.

=head2 Configuration

You can put some configuration items in ~/.gt/options to indicate where
the database is.

=over 

=item DB::mysql::dbname : the name of the database ("beancounter" by default)

=item DB::mysql::dbhost : the host of the database ("" = localhost by default)

=item DB::mysql::dbuser : the user account on the database

=item DB::mysql::dbpasswd : the password of the user account

=back

=head2 Functions

=over

=item C<< GT::DB::mysql->new() >>

=cut
sub new {
    my $type = shift;
    my $class = ref($type) || $type;

    GT::Conf::default("DB::mysql::dbname", "beancounter");
    GT::Conf::default("DB::mysql::dbhost", "");   #aka localhost
    GT::Conf::default("DB::mysql::dbuser", "");   #aka current user
    GT::Conf::default("DB::mysql::dbpasswd", ""); #aka user is already identified

    my $self = { 'dbname'   => GT::Conf::get("DB::mysql::dbname"),
    		 'dbhost'   => GT::Conf::get("DB::mysql::dbhost"), 
		 'dbuser'   => GT::Conf::get("DB::mysql::dbuser"), 
		 'dbpasswd' => GT::Conf::get("DB::mysql::dbpasswd"), 
		 @_
		};
		
    my $connect_string = 'dbi:mysql:dbname=' . $self->{'dbname'};
    if ($self->{'dbhost'}) {
	$connect_string .= "host=" . $self->{'dbhost'};
    }
    $self->{'_dbh'} = DBI->connect($connect_string, $self->{'dbuser'},
    		$self->{'dbpasswd'}) || die "Couldn't connect to database !\n";

    return bless $self, $class;
}

=item C<< $db->disconnect >>

Disconnects from the database.

=cut
sub disconnect {
    my $self = shift;
    $self->{'_dbh'}->disconnect;
    delete $self->{'prices'};
    delete $self->{'dates'};
}

=item C<< $db->get_prices($code) >>

Returns a GT::Prices object containing all known prices for the symbol $code.

=cut
sub get_prices {
    return get_last_prices(@_, -1);
}

=item C<< $db->get_last_prices($code, $limit) >>

Returns a GT::Prices object containing the $limit last known prices for
the symbol $code.

=cut
sub get_last_prices {
    my ($self, $code, $limit) = @_;

    my $q = GT::Prices->new($limit);
    $q->set_timeframe($DAY);

    my $sql = qq{ SELECT date, day_open, day_high, day_low, day_close, volume,
		    previous_close
    		  FROM stockprices where symbol="$code" ORDER BY date DESC };
    if ($limit > 0) {
	$sql .= "LIMIT $limit";
    }
    my $sth = $self->{'_dbh'}->prepare($sql) 
    	|| die $self->{'_dbh'}->errstr;
    $sth->execute() || die $self->{'_dbh'}->errstr;
    my $res;
    my @prices;
    while($res = $sth->fetchrow_arrayref)
    {
	if (! $res->[5])	# This is to suppress empty volume
	{			#   some stock may not have transaction,
	    $res->[5]=0;	#   the beancounter set it to NULL
	}			#   which causes a lot of warning in graphic.pl

	# This part is to avoid NULL opening price, some stock may not
	#   have opening price in yahoo finance page.  It causes ugly
	#   candle stick plotting.
	#   If prev close is between today's low/high, use it as opening
	#   if prev close is less than today's low, use today's low as opening
	#   If prev close is higher than today's high, use today's high
	if (! $res->[1])
	{
	    if ( $res->[6] < $res->[3])
	    {
		$res->[1]=$res->[3];	# use low
	    }
	    elsif ( $res->[6] > $res->[2])
	    {
		$res->[1]=$res->[2];	# use high
	    }
	    else
	    {
		$res->[1]=$res->[6];	# use prev close
	    }
	}

	if (! $res->[3])	# This is to avoid NULL low price
	{			#   I found stock without low price
	    $res->[3]=$res->[4];#   causes ugly candle stick plotting
	}			#   so, set it to close price

	push @prices, [ $res->[1], $res->[2], $res->[3], 
				$res->[4], $res->[5], $res->[0] ];
    }
    foreach $res (reverse @prices)
    {
	$q->add_prices($res);
    }
    return $q;
}

=item C<< $db->get_db_name($code) >>

Returns the name of the stock designated by $code.

=cut
sub get_db_name {
    my ($self, $code) = @_;

    my $sql = "SELECT name FROM stock WHERE symbol = '$code'";
    my $sth = $self->{'_dbh'}->prepare($sql) || die $self->{'_dbh'}->errstr;
    $sth->execute() || die $self->{'_dbh'}->errstr;
    my $res = $sth->fetchrow_arrayref;
    return $res->[0];
}

=pod

=back

=cut
1;
